// -*- Java -*-

// PopulateDB.java (14.5. 2003)

import java.lang.*;
import java.io.*;

import java.util.*;
import java.sql.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * PopulateDB class will run prepared sql scripts on a given db connection (set
 * up at application startup through one of the listeners).
 *
 * sql script files should be defined in a static member sqlScripts.
 *
 * good idea is to have one sql script per db table, each one starting with
 * 'DROP TABLE ...', 'CREATE TABLE ...' and then some 'INSERT ...' statements.
 *
 */

public class PopulateDB extends HttpServlet {

    private static final String sqlScriptsDir = "db_scripts";

    protected void service(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException {
	resp.setContentType("text/html");
	resp.setBufferSize(8192);
	PrintWriter out = resp.getWriter();

	out.println("<!DOCTYPE " +
		    "HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">" +
		    "<html>" +
		    "<head><title>" +
		    getClass().getName() +
		    "</title></head>" +
		    "<body>" +
		    "<h1>PopulateDB</h1>" +
		    "<pre>");

	Connection con =
	    (Connection) getServletContext().getAttribute("dbcon");
	if (getServletContext().getRealPath(sqlScriptsDir) != null &&
	    con != null) {
	    executeSqlScripts(out, con);
	}
	else {
	    out.println("Cannot execute sql scripts -- " +
			"cannot get the real paths of the scripts " +
			"(probably deployed as a war archive?).");
	}
	
	out.println("</pre></body></html>");
	out.close();
    }

    private void executeSqlScripts(PrintWriter out, Connection con) {
	File dir = new File(getServletContext().getRealPath(sqlScriptsDir));
	String[] files = dir.list();

	for (int i = 0; i < files.length; ++i) {
	    String sqlLine = null;
	    out.println("\nProcessing file: " + files[i]);
	    try {
		BufferedReader in = new BufferedReader(
					new FileReader(dir.getAbsolutePath() +
						       "/" + files[i]));
		while ((sqlLine = in.readLine()) != null) {
		    try {
			Statement sql = con.createStatement();
			sql.executeUpdate(sqlLine);
			out.println(sqlLine);
		    }
		    catch (SQLException err) {
			out.println("Cannot query: " + sqlLine);
		    }
		}
	    }
	    catch (IOException err) {
		out.println("IO error (file: " + files[i] + ")");
	    }
	}
    }

    public String getServletInfo() {
        return "The PopulateDB servlet initializes the database " +
	    "for the first use.";
    }

};
