// -*- Java -*-

// Cal.java (14.5. 2003)

package drdcal;

import java.lang.*;
import java.io.*;

import java.util.*;
import java.sql.*;

public class Cal {

    private Calendar calendar;
    private int daysInMonth;
    private Collection freeDays;
    private java.util.Date firstDay;
    private java.util.Date lastDay;
    private UserInfo user;

    public static String[] monthNames = new String[12];

    static {
	int i = 0;
	monthNames[i++] = "January";
	monthNames[i++] = "February";
	monthNames[i++] = "Macrh";
	monthNames[i++] = "April";
	monthNames[i++] = "May";
	monthNames[i++] = "June";
	monthNames[i++] = "July";
	monthNames[i++] = "August";
	monthNames[i++] = "September";
	monthNames[i++] = "October";
	monthNames[i++] = "November";
	monthNames[i++] = "December";
    }

    public Cal() {
	calendar = null;
	daysInMonth = 0;
	freeDays = null;
	firstDay = null;
	lastDay = null;
	user = null;
    }

    public synchronized void init(UserInfo user, Connection con) {
	calendar = new GregorianCalendar();
	this.user = user;
	getDays(con);
    }

    private synchronized void getDays(Connection con) {
	daysInMonth = calendar.getActualMaximum(Calendar.DATE);
	Calendar tmpCal = new GregorianCalendar();
	tmpCal.set(calendar.get(Calendar.YEAR),
		   calendar.get(Calendar.MONTH),
		   calendar.getActualMinimum(Calendar.DATE));
	firstDay = tmpCal.getTime();
	tmpCal.set(calendar.get(Calendar.YEAR),
		   calendar.get(Calendar.MONTH),
		   calendar.getActualMaximum(Calendar.DATE));
	lastDay = tmpCal.getTime();
	try {
	    String query = "SELECT DATE_FORMAT(day, '%e') AS day " +
		"FROM freeday " +
		"WHERE user_id = '" + user.getId() + "' AND " +
		"(TO_DAYS(FROM_UNIXTIME(" +
		(firstDay.getTime() / 1000) + ")) <= " +
		"TO_DAYS(day) AND " +
		"TO_DAYS(day) <= " +
		"TO_DAYS(FROM_UNIXTIME(" +
		(lastDay.getTime() / 1000) + "))) " +
		"ORDER BY day";
	    //System.err.println(query);
	    Statement sql = con.createStatement();
	    ResultSet res = sql.executeQuery(query);
	    freeDays = new ArrayList();
	    while (res.next()) {
		freeDays.add(new Integer(res.getInt("day")));
	    }
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	}
    }

    public synchronized int getDaysInMonth() { return daysInMonth; }

    public synchronized UserInfo getUser() { return user; }

    public synchronized Calendar getCalendar() { return calendar; }

    public synchronized Collection getFreeDays() { return freeDays; }
    public synchronized void setFreeDays(Collection days) { freeDays = days; }

    public synchronized String getMonthName() {
	return monthNames[calendar.get(Calendar.MONTH)] + " " +
	    calendar.get(Calendar.YEAR);
    }

    public synchronized java.util.Date getTime() {
	return calendar.getTime();
    }
    public synchronized void setTime(java.util.Date date, Connection con) {
	calendar.setTime(date);
	getDays(con);
    }

    public synchronized String toString() {
	return "Cal [" +
	    " time: " + getTime() +
	    ", year: " + calendar.get(Calendar.YEAR) +
	    ", month: " + calendar.get(Calendar.MONTH) +
	    ", daysInMonth: " + daysInMonth +
	    ", freeDays: " + freeDays +
	    ", firstDay: " + firstDay +
	    ", lastDay: " + lastDay +
	    " ]";
    }

    public synchronized void prevMonth(Connection con) {
	calendar.add(Calendar.MONTH, -1);
	getDays(con);
    }

    public synchronized void thisMonth(Connection con) {
	calendar.setTime(new java.util.Date());
	getDays(con);
    }

    public synchronized void nextMonth(Connection con) {
	calendar.add(Calendar.MONTH, 1);
	getDays(con);
    }

    public synchronized boolean contains(int day) {
 	Iterator i = freeDays.iterator();
	while (i.hasNext()) {
	    if (((Integer) i.next()).intValue() == day)
		return true;
	}
	return false;
    }

    public synchronized void saveMonthToDb(Connection con)
	throws SQLException {
	deleteMonth(con);
	insertMonth(con);
    }

    private synchronized void deleteMonth(Connection con) throws SQLException {
	String query = "DELETE FROM freeday " +
	    "WHERE user_id = '" + user.getId() + "' AND " +
	    "(TO_DAYS(FROM_UNIXTIME(" +
	    (firstDay.getTime() / 1000) + ")) <= " +
	    "TO_DAYS(day) AND " +
	    "TO_DAYS(day) <= " +
	    "TO_DAYS(FROM_UNIXTIME(" +
	    (lastDay.getTime() / 1000) + ")))";
	Statement sql = con.createStatement();
	sql.executeUpdate(query);	
    }

    private synchronized void insertMonth(Connection con) throws SQLException {
	String query = "INSERT INTO freeday VALUES (NULL, ?, ?)";
	PreparedStatement sql = con.prepareStatement(query);
	sql.setInt(2, user.getId());
	String datePart =
	    calendar.get(Calendar.YEAR) + "-" +
	    (calendar.get(Calendar.MONTH) + 1) + "-";
 	Iterator i = freeDays.iterator();
	while (i.hasNext()) {
	    int day = ((Integer) i.next()).intValue();
	    sql.setString(1, datePart + day);
	    sql.execute();
	}
    }

    public static void main(String[] args) {

	Connection con = null;
	UserInfo user = null;
	try {
	    Class.forName("com.mysql.jdbc.Driver").newInstance();
	    String dburi = "jdbc:mysql://localhost/test?" +
		"user=martin&password=martin";

	    con = DriverManager.getConnection(dburi);
	    user = new UserInfo("thorgal", con);
	    System.out.println(user);
	}
	catch (Exception err) {
	    err.printStackTrace();
	    System.exit(-1);
	}

	Cal cal = new Cal();
	cal.init(user, con);

	String cmd = null;
	BufferedReader in = new BufferedReader(
                                new InputStreamReader(System.in));
	try {
	    while (!(cmd = in.readLine()).equals("q")) {
		if (cmd.equals("p")) {
		    cal.prevMonth(con);
		}
		else if (cmd.equals("n")) {
		    cal.nextMonth(con);
		}
		System.out.println(cal);
	    }
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

};
