// -*- Java -*-

// TimeManager.java (13.5. 2003)

package drdcal;

import java.lang.*;
import java.io.*;

import javax.servlet.*;

import java.sql.*;
import java.util.*;

/**
 * <p>Starts up the timer {@link TimerThread}.
 *
 * <p>{@link TimerThread} is doing periodic tasks.
 *
 */

public class  TimeManager implements ServletContextListener {

    private TimerThread timer;;

    public TimeManager() {
	timer = null;
    }

    public void contextInitialized(ServletContextEvent sce) {
	Object con = sce.getServletContext().getAttribute("dbcon");
	Object email = sce.getServletContext().getAttribute("email");
	if (con != null && con instanceof Connection &&
	    email != null && email instanceof Email) {
	    timer = new TimerThread((Connection) con, (Email) email);
	    timer.start();
	    System.err.println("TimeManager: init ok");
	}
	else {
	    System.err.println("TimeManager: Cannot startup " +
			       "[dbcon: " + con + ", email: " + email + "]");
	}
    }

    public void contextDestroyed(ServletContextEvent sce) {
	if (timer != null) timer.interrupt();
	System.err.println("TimeManager: destroy ok");
    }
};


class TimerThread extends Thread {

    private Connection dbcon;
    private Email email;

    private final int SLEEP_INTERVAL = 1000 * 60 * 5;

    private final double LAST_LOGIN_INTERVAL = 48.0; // hours

    public TimerThread(Connection aDbcon, Email aEmail) {
	dbcon = aDbcon;
	email = aEmail;
    }

    public void run() {
	while (!isInterrupted()) {
	    checkLastLogin(LAST_LOGIN_INTERVAL);
	    checkCommonFreetime();
	    try {
		sleep(SLEEP_INTERVAL);
	    }
	    catch (InterruptedException err) {
		//err.printStackTrace();
		interrupt();
	    }
	    runningTest();
	}
	dbcon = null;
	email = null;
    }

    public String toString() {
	return getClass().getName() + " [" +
	    "interrupted: " + isInterrupted() +
	    "]";
    }

    private void runningTest() {
	System.err.println(this);
    }

    private void checkLastLogin(double hourInterval) {
	try {	    
	    String query = "SELECT username, email, " +
		"(IF(((UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_login)) / " +
		"(60 * 60)) > " + hourInterval + ", " +
		"1, 0)) AS sent_email " +
		"FROM userinfo " +
		"ORDER BY username ASC";
	    Statement sql = dbcon.createStatement();
	    ResultSet res = sql.executeQuery(query);
	    while (res.next()) {
		if (res.getInt("sent_email") == 1) {
		    String text = "Dear " + res.getString("username") + ",\n" +

			"this is message from DrD Calendar Mail Service.\n" +

			"You have not been logged in for more than " +
			((int) hourInterval) + " hours (more than " +
			((int) (hourInterval / 24)) + " day(s))!\n" +

			"Please point your Internet browser to DrD Calendar " +
			"application and input your free time.\n" +

			"Click at: [uri]\n";
		    email.send(res.getString("email"), text);
		}
	    }
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	}
    }

    private void checkCommonFreetime() {

	String query = null;
	Statement sql = null;
	ResultSet res = null;
	try {
	    sql = dbcon.createStatement();
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	    return;
	}
	/*
	// lock tables
	try {
	    query = "LOCK TABLE userinfo READ, freeday READ";
	    sql.executeUpdate(query);
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	    return;
	}
	*/
	try {
	    // user count

	    query = "SELECT COUNT(id) AS user_count FROM userinfo " +
		"WHERE deleted = 'NO'";
	    res = sql.executeQuery(query);
	    int userCount = Integer.MAX_VALUE;
	    if (res.first()) userCount = res.getInt("user_count");

	    // get emails

	    Collection emails = new ArrayList();
	    query = "SELECT email FROM userinfo " +
		"WHERE deleted = 'NO'";
	    res = sql.executeQuery(query);
	    while (res.next()) {
		emails.add(res.getString("email"));
	    }

	    // find common free time

	    query = "SELECT COUNT(userinfo.id) AS user_count, day " +
		"FROM userinfo, freeday " +
		"WHERE freeday.user_id = userinfo.id AND " +
		"userinfo.deleted = 'NO' AND " +
		"day > NOW() AND " +
		// "user_count = '" + userCount + "'" +
		"1 " +
		"GROUP BY day ORDER BY day";
	    res = sql.executeQuery(query);
	    while (res.next()) {
		int currentCount = res.getInt("user_count");
		if (currentCount == userCount) {
		    String msg = "Dear drd players,\n" +

			"this is message from DrD Calendar Mail Service.\n" +

			"Day " + res.getString("day") + " was marked by all " +
			"current users as playable.\n" +

			"Please notice that and contact other players.\n";
		    email.send(emails, msg);
		}
	    }
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	}
	/*
	// unlock tables
	try {
	    query = "UNLOCK TABLES";
	    sql = dbcon.createStatement();
	    sql.executeUpdate(query);
	}
	catch (SQLException sqlex) {
	    sqlex.printStackTrace();
	}
	*/
    }

};
