// -*- Java -*-

// UserInfo.java (13.5.2003)

package drdcal;

import java.lang.*;
import java.io.*;

import java.util.*;
import java.sql.*;

public class UserInfo {

    int id;
    String username;
    String email;
    java.util.Date lastLogin;
    boolean deleted;

    public UserInfo() {
	id = 0;
	username = "";
	email = "";
	lastLogin = new java.util.Date();
	deleted = false;
    }

    public UserInfo(String aName, Connection dbcon) throws SQLException {
	String query = "SELECT id, username, email, " +
	    "UNIX_TIMESTAMP(last_login) * 1000 AS last_login, " +
	    "deleted " +
	    "FROM userinfo WHERE username = '" + aName + "'";
	Statement sql = dbcon.createStatement();
	ResultSet res = sql.executeQuery(query);
	if (res.next()) {
	    id = res.getInt("id");
	    username = res.getString("username");
	    email = res.getString("email");
	    lastLogin = new java.util.Date(res.getLong("last_login"));
	    if (res.getString("deleted").equals("YES")) deleted = true;
	    else deleted = false;
	}
    }

    public synchronized int getId() { return id; }
    public synchronized void setId(int aId) { id = aId; }

    public synchronized String getUsername() { return username; }
    public synchronized void setUsername(String aName) { username = aName; }

    public synchronized String getEmail() { return email; }
    public synchronized void setEmail(String aEmail) { email = aEmail; }

    public synchronized java.util.Date getLastLogin() { return lastLogin; }
    public synchronized void setLastLogin(java.util.Date aDate) {
	lastLogin = aDate; }

    public synchronized String toString() {
	return "UserInfo [" +
	    " id: " + id +
	    ", username: " + username +
	    ", email: " + email +
	    ", lastLogin: " + lastLogin +
	    ", deleted: " + deleted +
	    " ]";
    }

    public synchronized boolean emailOk() {
	return email.matches("\\w+@\\w+(\\.\\w+)*");
    }

    public synchronized boolean usernameOk(Connection con)
	throws SQLException {
	if (username.matches("\\w+")) {
	    String query = "SELECT username " +
		"FROM userinfo " +
		"WHERE username = '" + username + "' AND " +
		"id <> '" + id + "'";
	    Statement sql = con.createStatement();
	    ResultSet res = sql.executeQuery(query);
	    if (!res.first()) return true;
	}
	return false;
    }

    public synchronized void saveToDb(Connection con) throws SQLException {
	if (id == 0) { // insert
	    insert(con);
	}
	else { // update
	    update(con);
	}
    }

    private synchronized void insert(Connection con) throws SQLException {
	String query = "INSERT INTO userinfo VALUES " +
	    "(NULL, ?, ?, NULL, 'NO')";
	PreparedStatement sql = con.prepareStatement(query);
	sql.setString(1, username);
	sql.setString(2, email);
	sql.execute();
	ResultSet res = sql.executeQuery("SELECT LAST_INSERT_ID()");
	res.first();
	id = res.getInt(1);
    }

    private synchronized void update(Connection con) throws SQLException {
	String query = "UPDATE userinfo SET " +
	    "username = ?, email = ?, last_login = last_login " +
	    "WHERE id = '" + id + "'";
	PreparedStatement sql = con.prepareStatement(query);
	sql.setString(1, username);
	sql.setString(2, email);
	sql.execute();
    }

    public synchronized void delete(Connection con) throws SQLException {
	String query = "DELETE FROM userinfo WHERE id ='" + id + "'";
	Statement sql = con.createStatement();
	sql.executeUpdate(query);
    } 

    public synchronized void pseudoDelete(Connection con) throws SQLException {
	String query = "UPDATE userinfo SET deleted = 'YES' " +
	    "WHERE id ='" + id + "'";
	Statement sql = con.createStatement();
	sql.executeUpdate(query);
    } 

    public synchronized static Collection getAllUsers(Connection con)
	throws SQLException {
	String query = "SELECT id, username, email, " +
	    "DATE_FORMAT(last_login, \"%e.%c. %k:%i \") AS last_login " +
	    "FROM userinfo " +
	    "WHERE deleted = 'NO' " +
	    "ORDER BY username ASC";
	Statement sql = con.createStatement();
	ResultSet res = sql.executeQuery(query);
	Collection msgs = new ArrayList();
	while (res.next()) {
	    Map msg = new Hashtable();
	    msg.put("id", res.getString("id"));
	    msg.put("username", res.getString("username"));
	    msg.put("email", res.getString("email"));
	    msg.put("lastLogin", res.getString("last_login"));
	    msgs.add(msg);
	}
	return msgs;
    }

};
